
## library which can be included in any guidepost like module
module CommonGuidepost
  # expects that these methods/properties are defined: coordinates
    
  ### methods for finding geographocally close guideposts ##############################
  def unambiguous_sez_guidepost; SezGuidepost.unambiguous(self.coordinates) end
  def unambiguous_osm_guidepost; OsmGuidepost.unambiguous(self.coordinates) end
  def has_unambiguous_sez_guidepost?; !unambiguous_sez_guidepost.nil? end
  def has_unambiguous_osm_guidepost?; !unambiguous_osm_guidepost.nil? end

  # distance to another_guidepost (not necesarilly of the same guidepost type)
  def distance_to(another_guidepost) 
    Map.distance(self.coordinates,another_guidepost.coordinates)
  end
      

  module ClassMethods # tohle je nutne protoze def self.metoda nefunguje, viz nize. BTW v tomto modulu self neni instance ale trida
        
    ### methods for finding geographocally close guideposts ##############################
    
    # get neighbours sorted by distance as [dist, guidepost] pairs
    def nearby(coordinates, max_meters = 999)
      return [] if coordinates[0].nil? or coordinates[1].nil?
      nearby_guess(coordinates, max_meters)
        .collect { |guidepost| [Map.distance(coordinates, guidepost.coordinates), guidepost]}
        .sort_by{|pair| pair[0] }
        .reject{ |pair| pair[0] > max_meters}
    end  
    
    # returns two nearest guideposts to coordinates as [{distance: dist1, guidepost: nearest},{distance: dist2, guidepost: 2ndnearest}]
    # if any of them is farther that max_meters, than {distance:Float::INFINITY, guidepost:nil} is returned in the respective part
    def two_nearest(coordinates, max_meters = 999)
      pom = nearby(coordinates, max_meters) 
      if pom.size < 2 # jeli pole kratke dopln "nilama"
        pom.push( [Float::INFINITY, nil], [Float::INFINITY, nil])
      end
      
      [{distance: pom[0][0], guidepost: pom[0][1]}, {distance: pom[1][0], guidepost: pom[1][1]}] 
    end
  
    # returns nil if there is no single unambiguous guidepost to coordinates, otherwise returns the unambiguous guidepost itself.
    # clear means that there is ONE guidepost TRESHOLD_DIST1 near AND no other ones TRESHOLD_DIST2 near
    def unambiguous(coordinates, treshold_dist1=40, treshold_dist2=80)
      nearest = two_nearest(coordinates, treshold_dist2 )
      if (nearest.second[:distance] >= treshold_dist2) and (nearest.first[:distance]<= treshold_dist1)
        return nearest.first[:guidepost]
      else
        return nil
      end
    end
    
    # fast way to get neighbours. Uses max coord metrics, only returns ActiveRelation to get the guideposts
    def nearby_guess(coordinates, max_meters = 999)
      self.where(lat: coordinates[0].plusminus(max_meters/Map.min_dlat), lon: coordinates[1].plusminus(max_meters/Map.min_dlon) )
    end

  end

  # tohle je nutne protoze def self.metoda nefunguje
  # see https://stackoverflow.com/questions/10692961/inheriting-class-methods-from-modules-mixins-in-ruby
  def self.included(base); base.extend ClassMethods end  

end
